"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tsd_1 = require("tsd");
const __1 = require("..");
const symbols_1 = require("../symbols");
const { pages, usePagesRouter } = __1.nextPages({
    index: __1.$route,
    about: __1.$route,
    users: {
        index: __1.$route,
        '[userId]': {
            index: __1.$route,
            posts: {
                '[...postId]': {
                    index: __1.$route,
                },
            },
            settings: {
                '[[...path]]': __1.$route,
                index: __1.$route,
                lang: __1.$route,
            },
        },
    },
    users2: {
        test: __1.$route,
    },
});
describe(__1.nextPages.name, () => {
    test.each([
        [pages[symbols_1.$index], ''],
        [pages.index, '/'],
        [pages.about, '/about'],
        [pages.users[symbols_1.$index], '/users'],
        [pages.users.index, '/users'],
        [pages.users(null)[symbols_1.$index], '/users/[userId]'],
        [pages.users(null).index, '/users/[userId]'],
        [pages.users('123')[symbols_1.$index], '/users/123'],
        [pages.users('123').index, '/users/123'],
        [pages.users('123').posts('456').index, '/users/123/posts/456'],
        [pages.users('123').posts('456', '789').index, '/users/123/posts/456/789'],
        [
            pages.users(null).posts(null)[symbols_1.$index],
            '/users/[userId]/posts/[...postId]',
        ],
        [pages.users(null).posts(null).index, '/users/[userId]/posts/[...postId]'],
        [pages.users('123').posts(null).index, '/users/123/posts/[...postId]'],
        [pages.users('123').settings.index, '/users/123/settings'],
        [pages.users('123').settings.lang, '/users/123/settings/lang'],
        [pages.users('123').settings('a', 'b'), '/users/123/settings/a/b'],
        [pages.users('123').settings(), '/users/123/settings'],
        [pages.users('123').settings(null), '/users/123/settings/[[...path]]'],
    ])('%p', (actual, expected) => expect(actual).toBe(expected));
    // @ts-expect-error: non dynamic route
    void (() => pages.about());
    // @ts-expect-error: pass 0 segments to dynamic route
    void (() => pages.users());
    // @ts-expect-error: pass multiple segments to dynamic route
    void (() => pages.users('123', '456'));
    // @ts-expect-error: pass 0 segments to catch all route
    void (() => pages.users('123').posts().index);
});
describe('isSubpathOf', () => {
    test.each([
        ['/users', pages.users, true],
        ['/users/[userId]', pages.users, true],
        ['/users/[userId]/posts', pages.users, true],
        ['/users2/test', pages.users, false],
        ['/users', pages.users2, false],
        ['/about', pages.users, false],
    ])('%p', (pathname, routeOptions, expected) => expect(__1.isSubpathOf(pathname, routeOptions)).toBe(expected));
});
test('usePagesRouter', () => {
    tsd_1.expectType({});
    // @ts-expect-error wrong key
    tsd_1.expectType({});
});
