"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isSubpathOf = exports.nextPages = void 0;
const router_1 = require("next/router");
const symbols_1 = require("./symbols");
const utils_1 = require("./utils");
const mapRouteValue = (path, route) => {
    return (route === symbols_1.$route
        ? path || '/'
        : mapRouteOptions(path, route));
};
const mapRouteOptions = (prevPath, options) => {
    var _a;
    const optionEntries = Object.entries(options);
    const [dynamicRoutePath, dynamicRoute] = (_a = optionEntries.find(([key]) => utils_1.isDynamicRouteKey(key))) !== null && _a !== void 0 ? _a : [];
    const dynamicRouteFunction = dynamicRoute
        ? (...key) => mapRouteValue(utils_1.joinPath(prevPath, key[0] === null ? [dynamicRoutePath] : key), dynamicRoute)
        : {};
    const mappedOptions = Object.fromEntries(optionEntries.map(([key, route]) => [key, mapRouteValue(utils_1.joinPath(prevPath, [key]), route)]));
    return Object.assign(dynamicRouteFunction, mappedOptions, {
        [symbols_1.$index]: prevPath,
    });
};
const nextPages = (options) => {
    const pages = mapRouteOptions('', options);
    const usePagesRouter = () => router_1.useRouter();
    return { pages, usePagesRouter };
};
exports.nextPages = nextPages;
const isSubpathOf = (pathname, routeOptions) => {
    const directoryPath = routeOptions[symbols_1.$index];
    return pathname === directoryPath || pathname.startsWith(directoryPath + '/');
};
exports.isSubpathOf = isSubpathOf;
